/*
===============================================================================

	FILE:	vector.cpp
	
	CONTENTS:
	
		Face Fixer - SIGGRAPH 2000 demonstration software
	
	PROGRAMMERS:
	
		Martin Isenburg@cs.unc.edu
	
	COPYRIGHT:
	
		Copyright (C) 2000  Martin Isenburg (isenburg@cs.unc.edu)
		
		This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	
	CHANGE HISTORY:
	
		28 April 2000 -- Martin - finalized for SIGGRAPH submission.
	
===============================================================================
*/
#include "vector.h"

#include <iostream.h>

FFvector::FFvector()
{
  current_size = 0;
  current_capacity = 1000;
  data = (void**) new int[1000];
}

FFvector::~FFvector()
{
  delete [] data;
}

int FFvector::isEmpty()
{
  if (current_size == 0)
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

void FFvector::ensureCapacity(int c)
{
  if (current_capacity < c)
    {
      void** temp = (void**) new int[c];
      for (int i = 0; i < current_size; i++)
	{
	  temp[i] = data[i];
	}
      current_capacity = c;
      delete [] data;
      data = temp;
    }
}

int FFvector::size() const
{
  return current_size;
}

void FFvector::setSize(int s)
{
  ensureCapacity(s);
  current_size = s;
}

void* FFvector::elementAt(int p) const
{
  if ((p < 0) || (p >= current_size))
    {
      return 0;
    }
  else
    {
      return data[p];
    }
}

void* FFvector::lastElement()
{
  if (current_size == 0)
    {
      return 0;
    }
  else
    {
      return data[current_size-1];
    }
}

void FFvector::addElement(void* e)
{
  checkCapacity();
  data[current_size] = e;
  current_size++;
}

void FFvector::setElementAt(void* e, int p)
{
  if ((p < 0) || (p >= current_size))
    {
      cerr << "ERROR in FFvector::setElementAt" << endl;
    }
  else
    {
      data[p] = e;
    }
}

void FFvector::removeElement(void* e)
{
  int p = -1;
  for (int i = current_size-1; i >= 0; i--)
    {
      if (e == data[i])
	{
	  p = i;
	  break;
	}
    }
  
  if (p == -1)
  {
    return;
  }
  
  for (i = p; i < current_size-1; i++)
    {
      data[i] = data[i+1];
    }
  current_size--;
}

void FFvector::removeElementAt(int p)
{
  if ((p < 0) || (p >= current_size))
    {
      return;
  }
  
  for (int i = p; i < current_size-1; i++)
    {
      data[i] = data[i+1];
    }
  current_size--;
}

void FFvector::removeLastElement()
{
  if (current_size > 0)
    {
      current_size--;
    }
}

void FFvector::removeAllElements()
{
  current_size = 0;
}

void FFvector::checkCapacity()
{
  if (current_capacity == current_size)
    {
      ensureCapacity(current_capacity*2);
    }
}
